@@ -1,5 +1,5 @@
 import {Component, OnInit} from "@angular/core";
-import {Ng2BalloonMsgService} from "@synerty/ng2-balloon-msg";
+import { BalloonMsgService } from "@synerty/peek-plugin-base-js"
 import {
     ComponentLifecycleEventEmitter,
     extend,
@@ -24,7 +24,7 @@ export class EditObjectTypeComponent extends ComponentLifecycleEventEmitter {
 
     loader: TupleLoader;
 
-    constructor(private balloonMsg: Ng2BalloonMsgService,
+    constructor(private balloonMsg: BalloonMsgService,
                 vortexService: VortexService) {
         super();
 
@@ -49,4 +49,4 @@ export class EditObjectTypeComponent extends ComponentLifecycleEventEmitter {
     }
 
 
-}
\ No newline at end of file
+}
