@@ -1,5 +1,5 @@
 import {Component, OnInit} from "@angular/core";
-import {Ng2BalloonMsgService} from "@synerty/ng2-balloon-msg";
+import { BalloonMsgService } from "@synerty/peek-plugin-base-js"
 import {
     extend,
     VortexService,
@@ -23,7 +23,7 @@ export class EditPropertyComponent extends ComponentLifecycleEventEmitter {
 
     loader: TupleLoader;
 
-    constructor(private balloonMsg: Ng2BalloonMsgService,
+    constructor(private balloonMsg: BalloonMsgService,
                 vortexService: VortexService) {
         super();
 
@@ -48,4 +48,4 @@ export class EditPropertyComponent extends ComponentLifecycleEventEmitter {
     }
 
 
-}
\ No newline at end of file
+}
